@@ -24,13 +24,13 @@ import itertools
 
 from gensim import utils
 
-if sys.version_info[0] >= 3:
-    unicode = str
-
 from six import PY3, iteritems, iterkeys, itervalues, string_types
 from six.moves import xrange
 from six.moves import zip as izip
 
+if sys.version_info[0] >= 3:
+    unicode = str
+
 
 logger = logging.getLogger('gensim.corpora.dictionary')
 
@@ -46,30 +46,27 @@ class Dictionary(utils.SaveLoad, Mapping):
         """
         If `documents` are given, use them to initialize Dictionary (see `add_documents()`).
         """
-        self.token2id = {} # token -> tokenId
-        self.id2token = {} # reverse mapping for token2id; only formed on request, to save memory
-        self.dfs = {} # document frequencies: tokenId -> in how many documents this token appeared
+        self.token2id = {}  # token -> tokenId
+        self.id2token = {}  # reverse mapping for token2id; only formed on request, to save memory
+        self.dfs = {}  # document frequencies: tokenId -> in how many documents this token appeared
 
-        self.num_docs = 0 # number of documents processed
-        self.num_pos = 0 # total number of corpus positions
-        self.num_nnz = 0 # total number of non-zeroes in the BOW matrix
+        self.num_docs = 0  # number of documents processed
+        self.num_pos = 0  # total number of corpus positions
+        self.num_nnz = 0  # total number of non-zeroes in the BOW matrix
 
         if documents is not None:
             self.add_documents(documents, prune_at=prune_at)
 
-
     def __getitem__(self, tokenid):
         if len(self.id2token) != len(self.token2id):
             # the word->id mapping has changed (presumably via add_documents);
             # recompute id->word accordingly
-            self.id2token = dict((v, k) for k, v in iteritems(self.token2id))
-        return self.id2token[tokenid] # will throw for non-existent ids
-
+            self.id2token = utils.revdict(self.token2id)
+        return self.id2token[tokenid]  # will throw for non-existent ids
 
     def __iter__(self):
         return iter(self.keys())
 
-
     if PY3:
         # restore Py2-style dict API
         iterkeys = __iter__
@@ -80,29 +77,24 @@ class Dictionary(utils.SaveLoad, Mapping):
         def itervalues(self):
             return self.values()
 
-
     def keys(self):
         """Return a list of all token ids."""
         return list(self.token2id.values())
 
-
     def __len__(self):
         """
         Return the number of token->id mappings in the dictionary.
         """
         return len(self.token2id)
 
-
     def __str__(self):
         some_keys = list(itertools.islice(iterkeys(self.token2id), 5))
         return "Dictionary(%i unique tokens: %s%s)" % (len(self), some_keys, '...' if len(self) > 5 else '')
 
-
     @staticmethod
     def from_documents(documents):
         return Dictionary(documents=documents)
 
-
     def add_documents(self, documents, prune_at=2000000):
         """
         Update dictionary from a collection of documents. Each document is a list
@@ -124,11 +116,12 @@ class Dictionary(utils.SaveLoad, Mapping):
                 logger.info("adding document #%i to %s", docno, self)
 
             # update Dictionary with the document
-            self.doc2bow(document, allow_update=True) # ignore the result, here we only care about updating token ids
-
-        logger.info("built %s from %i documents (total %i corpus positions)",
-                     self, self.num_docs, self.num_pos)
+            self.doc2bow(document, allow_update=True)  # ignore the result, here we only care about updating token ids
 
+        logger.info(
+            "built %s from %i documents (total %i corpus positions)",
+            self, self.num_docs, self.num_pos
+        )
 
     def doc2bow(self, document, allow_update=False, return_missing=False):
         """
@@ -155,14 +148,14 @@ class Dictionary(utils.SaveLoad, Mapping):
 
         token2id = self.token2id
         if allow_update or return_missing:
-            missing = dict((w, freq) for w, freq in iteritems(counter) if w not in token2id)
+            missing = {w: freq for w, freq in iteritems(counter) if w not in token2id}
             if allow_update:
                 for w in missing:
                     # new id = number of ids made so far;
                     # NOTE this assumes there are no gaps in the id sequence!
                     token2id[w] = len(token2id)
 
-        result = dict((token2id[w], freq) for w, freq in iteritems(counter) if w in token2id)
+        result = {token2id[w]: freq for w, freq in iteritems(counter) if w in token2id}
 
         if allow_update:
             self.num_docs += 1
@@ -180,15 +173,16 @@ class Dictionary(utils.SaveLoad, Mapping):
         else:
             return result
 
-
-    def filter_extremes(self, no_below=5, no_above=0.5, keep_n=100000):
+    def filter_extremes(self, no_below=5, no_above=0.5, keep_n=100000, keep_tokens=None):
         """
         Filter out tokens that appear in
 
         1. less than `no_below` documents (absolute number) or
         2. more than `no_above` documents (fraction of total corpus size, *not*
            absolute number).
-        3. after (1) and (2), keep only the first `keep_n` most frequent tokens (or
+        3. if tokens are given in keep_tokens (list of strings), they will be kept regardless of
+           the `no_below` and `no_above` settings
+        4. after (1), (2) and (3), keep only the first `keep_n` most frequent tokens (or
            keep all if `None`).
 
         After the pruning, shrink resulting gaps in word ids.
@@ -196,23 +190,53 @@ class Dictionary(utils.SaveLoad, Mapping):
         **Note**: Due to the gap shrinking, the same word may have a different
         word id before and after the call to this function!
         """
-        no_above_abs = int(no_above * self.num_docs) # convert fractional threshold to absolute threshold
+        no_above_abs = int(no_above * self.num_docs)  # convert fractional threshold to absolute threshold
 
         # determine which tokens to keep
-        good_ids = (v for v in itervalues(self.token2id)
-                      if no_below <= self.dfs.get(v, 0) <= no_above_abs)
+        if keep_tokens:
+            keep_ids = [self.token2id[v] for v in keep_tokens if v in self.token2id]
+            good_ids = (
+                v for v in itervalues(self.token2id)
+                if no_below <= self.dfs.get(v, 0) <= no_above_abs or v in keep_ids
+            )
+        else:
+            good_ids = (
+                v for v in itervalues(self.token2id)
+                if no_below <= self.dfs.get(v, 0) <= no_above_abs
+            )
         good_ids = sorted(good_ids, key=self.dfs.get, reverse=True)
         if keep_n is not None:
             good_ids = good_ids[:keep_n]
-        bad_words = [(self[id], self.dfs.get(id, 0)) for id in set(self).difference(good_ids)]
+        bad_words = [(self[idx], self.dfs.get(idx, 0)) for idx in set(self).difference(good_ids)]
         logger.info("discarding %i tokens: %s...", len(self) - len(good_ids), bad_words[:10])
-        logger.info("keeping %i tokens which were in no less than %i and no more than %i (=%.1f%%) documents",
-            len(good_ids), no_below, no_above_abs, 100.0 * no_above)
+        logger.info(
+            "keeping %i tokens which were in no less than %i and no more than %i (=%.1f%%) documents",
+            len(good_ids), no_below, no_above_abs, 100.0 * no_above
+        )
 
         # do the actual filtering, then rebuild dictionary to remove gaps in ids
         self.filter_tokens(good_ids=good_ids)
-        logger.info("resulting dictionary: %s" % self)
+        logger.info("resulting dictionary: %s", self)
 
+    def filter_n_most_frequent(self, remove_n):
+        """
+        Filter out the 'remove_n' most frequent tokens that appear in the documents.
+
+        After the pruning, shrink resulting gaps in word ids.
+
+        **Note**: Due to the gap shrinking, the same word may have a different
+        word id before and after the call to this function!
+        """
+        # determine which tokens to keep
+        most_frequent_ids = (v for v in itervalues(self.token2id))
+        most_frequent_ids = sorted(most_frequent_ids, key=self.dfs.get, reverse=True)
+        most_frequent_ids = most_frequent_ids[:remove_n]
+        # do the actual filtering, then rebuild dictionary to remove gaps in ids
+        most_frequent_words = [(self[idx], self.dfs.get(idx, 0)) for idx in most_frequent_ids]
+        logger.info("discarding %i tokens: %s...", len(most_frequent_ids), most_frequent_words[:10])
+
+        self.filter_tokens(bad_ids=most_frequent_ids)
+        logger.info("resulting dictionary: %s", self)
 
     def filter_tokens(self, bad_ids=None, good_ids=None):
         """
@@ -223,23 +247,14 @@ class Dictionary(utils.SaveLoad, Mapping):
         """
         if bad_ids is not None:
             bad_ids = set(bad_ids)
-            self.token2id = dict((token, tokenid)
-                                 for token, tokenid in iteritems(self.token2id)
-                                 if tokenid not in bad_ids)
-            self.dfs = dict((tokenid, freq)
-                            for tokenid, freq in iteritems(self.dfs)
-                            if tokenid not in bad_ids)
+            self.token2id = {token: tokenid for token, tokenid in iteritems(self.token2id) if tokenid not in bad_ids}
+            self.dfs = {tokenid: freq for tokenid, freq in iteritems(self.dfs) if tokenid not in bad_ids}
         if good_ids is not None:
             good_ids = set(good_ids)
-            self.token2id = dict((token, tokenid)
-                                 for token, tokenid in iteritems(self.token2id)
-                                 if tokenid in good_ids)
-            self.dfs = dict((tokenid, freq)
-                            for tokenid, freq in iteritems(self.dfs)
-                            if tokenid in good_ids)
+            self.token2id = {token: tokenid for token, tokenid in iteritems(self.token2id) if tokenid in good_ids}
+            self.dfs = {tokenid: freq for tokenid, freq in iteritems(self.dfs) if tokenid in good_ids}
         self.compactify()
 
-
     def compactify(self):
         """
         Assign new word ids to all words.
@@ -254,22 +269,24 @@ class Dictionary(utils.SaveLoad, Mapping):
         idmap = dict(izip(itervalues(self.token2id), xrange(len(self.token2id))))
 
         # reassign mappings to new ids
-        self.token2id = dict((token, idmap[tokenid]) for token, tokenid in iteritems(self.token2id))
+        self.token2id = {token: idmap[tokenid] for token, tokenid in iteritems(self.token2id)}
         self.id2token = {}
-        self.dfs = dict((idmap[tokenid], freq) for tokenid, freq in iteritems(self.dfs))
-
+        self.dfs = {idmap[tokenid]: freq for tokenid, freq in iteritems(self.dfs)}
 
     def save_as_text(self, fname, sort_by_word=True):
         """
         Save this Dictionary to a text file, in format:
+        `num_docs`
         `id[TAB]word_utf8[TAB]document frequency[NEWLINE]`. Sorted by word,
         or by decreasing word frequency.
 
         Note: text format should be use for corpus inspection. Use `save`/`load`
         to store in binary format (pickle) for improved performance.
         """
-        logger.info("saving dictionary mapping to %s" % fname)
+        logger.info("saving dictionary mapping to %s", fname)
         with utils.smart_open(fname, 'wb') as fout:
+            numdocs_line = "%d\n" % self.num_docs
+            fout.write(utils.to_utf8(numdocs_line))
             if sort_by_word:
                 for token, tokenid in sorted(iteritems(self.token2id)):
                     line = "%i\t%s\t%i\n" % (tokenid, token, self.dfs.get(tokenid, 0))
@@ -279,7 +296,6 @@ class Dictionary(utils.SaveLoad, Mapping):
                     line = "%i\t%s\t%i\n" % (tokenid, self[tokenid], freq)
                     fout.write(utils.to_utf8(line))
 
-
     def merge_with(self, other):
         """
         Merge another dictionary into this dictionary, mapping same tokens to the
@@ -312,20 +328,19 @@ class Dictionary(utils.SaveLoad, Mapping):
             old2new[other_id] = new_id
             try:
                 self.dfs[new_id] += other.dfs[other_id]
-            except:
+            except Exception:
                 # `other` isn't a Dictionary (probably just a dict) => ignore dfs, keep going
                 pass
         try:
             self.num_docs += other.num_docs
             self.num_nnz += other.num_nnz
             self.num_pos += other.num_pos
-        except:
+        except Exception:
             pass
 
         import gensim.models
         return gensim.models.VocabTransform(old2new)
 
-
     @staticmethod
     def load_from_text(fname):
         """
@@ -336,6 +351,13 @@ class Dictionary(utils.SaveLoad, Mapping):
         with utils.smart_open(fname) as f:
             for lineno, line in enumerate(f):
                 line = utils.to_unicode(line)
+                if lineno == 0:
+                    if line.strip().isdigit():
+                        # Older versions of save_as_text may not write num_docs on first line.
+                        result.num_docs = int(line.strip())
+                        continue
+                    else:
+                        logging.warning("Text does not contain num_docs on the first line.")
                 try:
                     wordid, word, docfreq = line[:-1].split('\t')
                 except Exception:
@@ -348,7 +370,6 @@ class Dictionary(utils.SaveLoad, Mapping):
                 result.dfs[wordid] = int(docfreq)
         return result
 
-
     @staticmethod
     def from_corpus(corpus, id2word=None):
         """
@@ -369,7 +390,7 @@ class Dictionary(utils.SaveLoad, Mapping):
         max_id = -1
         for docno, document in enumerate(corpus):
             if docno % 10000 == 0:
-                logger.info("adding document #%i to %s" % (docno, result))
+                logger.info("adding document #%i to %s", docno, result)
             result.num_docs += 1
             result.num_nnz += len(document)
             for wordid, word_freq in document:
@@ -379,16 +400,16 @@ class Dictionary(utils.SaveLoad, Mapping):
 
         if id2word is None:
             # make sure length(result) == get_max_id(corpus) + 1
-            result.token2id = dict((unicode(i), i) for i in xrange(max_id + 1))
+            result.token2id = {unicode(i): i for i in xrange(max_id + 1)}
         else:
             # id=>word mapping given: simply copy it
-            result.token2id = dict((utils.to_unicode(token), id) for id, token in iteritems(id2word))
-        for id in itervalues(result.token2id):
+            result.token2id = {utils.to_unicode(token): idx for idx, token in iteritems(id2word)}
+        for idx in itervalues(result.token2id):
             # make sure all token ids have a valid `dfs` entry
-            result.dfs[id] = result.dfs.get(id, 0)
+            result.dfs[idx] = result.dfs.get(idx, 0)
 
-        logger.info("built %s from %i documents (total %i corpus positions)" %
-                     (result, result.num_docs, result.num_pos))
+        logger.info(
+            "built %s from %i documents (total %i corpus positions)",
+            result, result.num_docs, result.num_pos
+        )
         return result
-
-#endclass Dictionary
