@@ -18,7 +18,9 @@ of each document.
 """
 
 import logging
-import shelve
+import six
+
+import numpy
 
 from gensim import interfaces, utils
 
@@ -46,8 +48,10 @@ class IndexedCorpus(interfaces.CorpusABC):
             if index_fname is None:
                 index_fname = utils.smart_extension(fname, '.index')
             self.index = utils.unpickle(index_fname)
-            logger.info("loaded corpus index from %s" % index_fname)
-        except:
+            # change self.index into a numpy.ndarray to support fancy indexing
+            self.index = numpy.asarray(self.index)
+            logger.info("loaded corpus index from %s", index_fname)
+        except Exception:
             self.index = None
         self.length = None
 
@@ -65,6 +69,7 @@ class IndexedCorpus(interfaces.CorpusABC):
            each saved document,
         * the `docbyoffset(offset)` method, which returns a document
           positioned at `offset` bytes within the persistent storage (file).
+        * metadata if set to true will ensure that serialize will write out article titles to a pickle file.
 
         Example:
 
@@ -90,11 +95,14 @@ class IndexedCorpus(interfaces.CorpusABC):
                 offsets = serializer.save_corpus(fname, corpus, id2word, metadata=metadata)
 
         if offsets is None:
-            raise NotImplementedError("called serialize on class %s which doesn't support indexing!" %
-                serializer.__name__)
+            raise NotImplementedError("called serialize on class %s which doesn't support indexing!" % serializer.__name__)
 
         # store offsets persistently, using pickle
-        logger.info("saving %s index to %s" % (serializer.__name__, index_fname))
+        # we shouldn't have to worry about self.index being a numpy.ndarray as the serializer will return
+        # the offsets that are actually stored on disk - we're not storing self.index in any case, the
+        # load just needs to turn whatever is loaded from disk back into a ndarray - this should also ensure
+        # backwards compatibility
+        logger.info("saving %s index to %s", serializer.__name__, index_fname)
         utils.pickle(offsets, index_fname)
 
     def __len__(self):
@@ -106,18 +114,16 @@ class IndexedCorpus(interfaces.CorpusABC):
             return len(self.index)
         if self.length is None:
             logger.info("caching corpus length")
-            self.length = sum(1 for doc in self)
+            self.length = sum(1 for _ in self)
         return self.length
 
     def __getitem__(self, docno):
         if self.index is None:
             raise RuntimeError("cannot call corpus[docid] without an index")
 
-        if isinstance(docno, slice):
+        if isinstance(docno, (slice, list, numpy.ndarray)):
             return utils.SlicedCorpus(self, docno)
-
-        return self.docbyoffset(self.index[docno])
-
-
-
-# endclass IndexedCorpus
+        elif isinstance(docno, six.integer_types + (numpy.integer,)):
+            return self.docbyoffset(self.index[docno])
+        else:
+            raise ValueError('Unrecognised value for docno, use either a single integer, a slice or a numpy.ndarray')
