@@ -26,6 +26,12 @@ import numpy
 import scipy.sparse as sparse
 import time
 
+from six.moves import xrange
+
+import gensim
+from gensim.corpora import IndexedCorpus
+from gensim.interfaces import TransformedCorpus
+
 logger = logging.getLogger(__name__)
 
 #: Specifies which dtype should be used for serializing the shards.
@@ -37,13 +43,6 @@ except ImportError:
     logger.info('Could not import Theano, will use standard float for default ShardedCorpus dtype.')
 
 
-from six.moves import xrange
-
-import gensim
-from gensim.corpora import IndexedCorpus
-from gensim.interfaces import TransformedCorpus
-
-
 class ShardedCorpus(IndexedCorpus):
     """
     This corpus is designed for situations where you need to train a model
@@ -233,10 +232,9 @@ class ShardedCorpus(IndexedCorpus):
         self.current_shard = None    # The current shard itself (numpy ndarray)
         self.current_shard_n = None  # Current shard is the current_shard_n-th
         self.current_offset = None   # The index into the dataset which
-                                     # corresponds to index 0 of current shard
+        # corresponds to index 0 of current shard
 
-        logger.info('Initializing sharded corpus with prefix '
-                     '{0}'.format(output_prefix))
+        logger.info('Initializing sharded corpus with prefix %s', output_prefix)
         if (not os.path.isfile(output_prefix)) or overwrite:
             logger.info('Building from corpus...')
             self.init_shards(output_prefix, corpus, shardsize)
@@ -244,8 +242,7 @@ class ShardedCorpus(IndexedCorpus):
             # Save automatically, to facilitate re-loading
             # and retain information about how the corpus
             # was serialized.
-            logger.info('Saving ShardedCorpus object to '
-                         '{0}'.format(self.output_prefix))
+            logger.info('Saving ShardedCorpus object to %s', self.output_prefix)
             self.save()
         else:
             logger.info('Cloning existing...')
@@ -254,20 +251,21 @@ class ShardedCorpus(IndexedCorpus):
     def init_shards(self, output_prefix, corpus, shardsize=4096, dtype=_default_dtype):
         """Initialize shards from the corpus."""
 
-        if not gensim.utils.is_corpus(corpus):
-            raise ValueError('Cannot initialize shards without a corpus to read'
-                             ' from! (Got corpus type: {0})'.format(type(corpus)))
+        is_corpus, corpus = gensim.utils.is_corpus(corpus)
+        if not is_corpus:
+            raise ValueError(
+                "Cannot initialize shards without a corpus to read from! (Got corpus type: {0})".format(type(corpus))
+            )
 
         proposed_dim = self._guess_n_features(corpus)
         if proposed_dim != self.dim:
             if self.dim is None:
-                logger.info('Deriving dataset dimension from corpus: '
-                             '{0}'.format(proposed_dim))
+                logger.info('Deriving dataset dimension from corpus: %d', proposed_dim)
             else:
-                logger.warn('Dataset dimension derived from input corpus diffe'
-                             'rs from initialization argument, using corpus.'
-                             '(corpus {0}, init arg {1})'.format(proposed_dim,
-                                                                 self.dim))
+                logger.warning(
+                    "Dataset dimension derived from input corpus differs from initialization argument, "
+                    "using corpus. (corpus %d, init arg %d)", proposed_dim, self.dim
+                )
 
         self.dim = proposed_dim
         self.offsets = [0]
@@ -277,11 +275,10 @@ class ShardedCorpus(IndexedCorpus):
         logger.info('Running init from corpus.')
 
         for n, doc_chunk in enumerate(gensim.utils.grouper(corpus, chunksize=shardsize)):
-            logger.info('Chunk no. {0} at {1} s'.format(n, time.clock() - start_time))
+            logger.info('Chunk no. %d at %f s', n, time.clock() - start_time)
 
             current_shard = numpy.zeros((len(doc_chunk), self.dim), dtype=dtype)
-            logger.debug('Current chunk dimension: '
-                          '{0} x {1}'.format(len(doc_chunk), self.dim))
+            logger.debug('Current chunk dimension: %d x %d', len(doc_chunk), self.dim)
 
             for i, doc in enumerate(doc_chunk):
                 doc = dict(doc)
@@ -294,7 +291,7 @@ class ShardedCorpus(IndexedCorpus):
             self.save_shard(current_shard)
 
         end_time = time.clock()
-        logger.info('Built {0} shards in {1} s.'.format(self.n_shards, end_time - start_time))
+        logger.info('Built %d shards in %f s.', self.n_shards, end_time - start_time)
 
     def init_by_clone(self):
         """
@@ -309,11 +306,13 @@ class ShardedCorpus(IndexedCorpus):
 
         if temp.dim != self.dim:
             if self.dim is None:
-                logger.info('Loaded dataset dimension: {0}'.format(temp.dim))
+                logger.info('Loaded dataset dimension: %d', temp.dim)
             else:
-                logger.warn('Loaded dataset dimension differs from init arg '
-                             'dimension, using loaded dim. '
-                             '(loaded {0}, init {1})'.format(temp.dim, self.dim))
+                logger.warning(
+                    "Loaded dataset dimension differs from init arg dimension, "
+                    "using loaded dim. (loaded %d, init %d)",
+                    temp.dim, self.dim
+                )
 
         self.dim = temp.dim  # To be consistent with the loaded data!
 
@@ -328,7 +327,7 @@ class ShardedCorpus(IndexedCorpus):
         """
         new_shard = False
         if n is None:
-            n = self.n_shards # Saving the *next* one by default.
+            n = self.n_shards  # Saving the *next* one by default.
             new_shard = True
 
         if not filename:
@@ -344,8 +343,6 @@ class ShardedCorpus(IndexedCorpus):
         """
         Load (unpickle) the n-th shard as the "live" part of the dataset
         into the Dataset object."""
-        #logger.debug('ShardedCorpus loading shard {0}, '
-        #              'current shard: {1}'.format(n, self.current_shard_n))
 
         # No-op if the shard is already open.
         if self.current_shard_n == n:
@@ -387,22 +384,12 @@ class ShardedCorpus(IndexedCorpus):
                              ' supported.'.format(offset))
         return k
 
-        k = -1
-        for i, o in enumerate(self.offsets):
-            if o > offset:  # Condition should fire for every valid offset,
-                            # since the last offset is n_docs (one-past-end).
-                k = i - 1   # First offset is always 0, so i is at least 1.
-                break
-
-        return k
-
     def in_current(self, offset):
         """
         Determine whether the given offset falls within the current shard.
 
         """
-        return (self.current_offset <= offset) \
-                and (offset < self.offsets[self.current_shard_n + 1])
+        return (self.current_offset <= offset) and (offset < self.offsets[self.current_shard_n + 1])
 
     def in_next(self, offset):
         """
@@ -413,9 +400,8 @@ class ShardedCorpus(IndexedCorpus):
 
         """
         if self.current_shard_n == self.n_shards:
-            return False # There's no next shard.
-        return (self.offsets[self.current_shard_n + 1] <= offset) \
-               and (offset < self.offsets[self.current_shard_n + 2])
+            return False  # There's no next shard.
+        return (self.offsets[self.current_shard_n + 1] <= offset) and (offset < self.offsets[self.current_shard_n + 2])
 
     def resize_shards(self, shardsize):
         """
@@ -470,9 +456,7 @@ class ShardedCorpus(IndexedCorpus):
             for old_shard_n, old_shard_name in enumerate(old_shard_names):
                 os.remove(old_shard_name)
         except Exception as e:
-            logger.error('Exception occurred during old shard no. {0} '
-                          'removal: {1}.\nAttempting to at least move '
-                          'new shards in.'.format(old_shard_n, str(e)))
+            logger.error('Exception occurred during old shard no. %d removal: %s.\nAttempting to at least move new shards in.', old_shard_n, str(e))
         finally:
             # If something happens with cleaning up - try to at least get the
             # new guys in.
@@ -481,9 +465,8 @@ class ShardedCorpus(IndexedCorpus):
                     os.rename(new_shard_name, self._shard_name(shard_n))
             # If something happens when we're in this stage, we're screwed.
             except Exception as e:
-                print(e)
-                raise RuntimeError('Resizing completely failed for some reason.'
-                                   ' Sorry, dataset is probably ruined...')
+                logger.exception(e)
+                raise RuntimeError('Resizing completely failed for some reason. Sorry, dataset is probably ruined...')
             finally:
                 # Sets the new shard stats.
                 self.n_shards = n_new_shards
@@ -527,18 +510,21 @@ class ShardedCorpus(IndexedCorpus):
                 return self._guess_n_features(corpus.corpus)
         else:
             if not self.dim:
-                raise TypeError('Couldn\'t find number of features, '
-                                 'refusing to guess (dimension set to {0},'
-                                 'type of corpus: {1}).'.format(self.dim, type(corpus)))
+                raise TypeError(
+                    "Couldn't find number of features, refusing to guess "
+                    "(dimension set to {0}, type of corpus: {1})."
+                    .format(self.dim, type(corpus))
+                )
             else:
-                logger.warn('Couldn\'t find number of features, trusting '
-                             'supplied dimension ({0})'.format(self.dim))
+                logger.warning("Couldn't find number of features, trusting supplied dimension (%d)", self.dim)
                 n_features = self.dim
 
         if self.dim and n_features != self.dim:
-            logger.warn('Discovered inconsistent dataset dim ({0}) and '
-                         'feature count from corpus ({1}). Coercing to dimension'
-                         ' given by argument.'.format(self.dim, n_features))
+            logger.warning(
+                "Discovered inconsistent dataset dim (%d) and feature count from corpus (%d). "
+                "Coercing to dimension given by argument.",
+                self.dim, n_features
+            )
 
         return n_features
 
@@ -592,8 +578,7 @@ class ShardedCorpus(IndexedCorpus):
             start = offset.start
             stop = offset.stop
             if stop > self.n_docs:
-                raise IndexError('Requested slice offset {0} out of range'
-                                 ' ({1} docs)'.format(stop, self.n_docs))
+                raise IndexError('Requested slice offset {0} out of range ({1} docs)'.format(stop, self.n_docs))
 
             # - get range of shards over which to iterate
             first_shard = self.shard_by_offset(start)
@@ -604,16 +589,11 @@ class ShardedCorpus(IndexedCorpus):
                 # This fails on one-past
                 # slice indexing; that's why there's a code branch here.
 
-            #logger.debug('ShardedCorpus: Retrieving slice {0}: '
-            #              'shard {1}'.format((offset.start, offset.stop),
-            #                                 (first_shard, last_shard)))
-
             self.load_shard(first_shard)
 
             # The easy case: both in one shard.
             if first_shard == last_shard:
-                s_result = self.current_shard[start - self.current_offset:
-                                            stop - self.current_offset]
+                s_result = self.current_shard[start - self.current_offset: stop - self.current_offset]
                 # Handle different sparsity settings:
                 s_result = self._getitem_format(s_result)
 
@@ -621,11 +601,9 @@ class ShardedCorpus(IndexedCorpus):
 
             # The hard case: the slice is distributed across multiple shards
             # - initialize numpy.zeros()
-            s_result = numpy.zeros((stop - start, self.dim),
-                                   dtype=self.current_shard.dtype)
+            s_result = numpy.zeros((stop - start, self.dim), dtype=self.current_shard.dtype)
             if self.sparse_serialization:
-                s_result = sparse.csr_matrix((0, self.dim),
-                                             dtype=self.current_shard.dtype)
+                s_result = sparse.csr_matrix((0, self.dim), dtype=self.current_shard.dtype)
 
             # - gradually build it up. We will be using three set of start:stop
             #   indexes:
@@ -646,16 +624,14 @@ class ShardedCorpus(IndexedCorpus):
             #  - if in ending shard, these are from 0
             #    to (stop - current_offset)
             shard_start = start - self.current_offset
-            shard_stop = self.offsets[self.current_shard_n + 1] - \
-                         self.current_offset
+            shard_stop = self.offsets[self.current_shard_n + 1] - self.current_offset
 
-            #s_result[result_start:result_stop] = self.current_shard[
+            # s_result[result_start:result_stop] = self.current_shard[
             #                                         shard_start:shard_stop]
-            s_result = self.__add_to_slice(s_result, result_start, result_stop,
-                                           shard_start, shard_stop)
+            s_result = self.__add_to_slice(s_result, result_start, result_stop, shard_start, shard_stop)
 
             # First and last get special treatment, these are in between
-            for shard_n in xrange(first_shard+1, last_shard):
+            for shard_n in xrange(first_shard + 1, last_shard):
                 self.load_shard(shard_n)
 
                 result_start = result_stop
@@ -663,9 +639,7 @@ class ShardedCorpus(IndexedCorpus):
                 shard_start = 0
                 shard_stop = self.shardsize
 
-                s_result = self.__add_to_slice(s_result, result_start,
-                                               result_stop, shard_start,
-                                               shard_stop)
+                s_result = self.__add_to_slice(s_result, result_start, result_stop, shard_start, shard_stop)
 
             # Last shard
             self.load_shard(last_shard)
@@ -674,9 +648,7 @@ class ShardedCorpus(IndexedCorpus):
             shard_start = 0
             shard_stop = stop - self.current_offset
 
-            s_result = self.__add_to_slice(s_result, result_start, result_stop,
-                                           shard_start, shard_stop)
-
+            s_result = self.__add_to_slice(s_result, result_start, result_stop, shard_start, shard_stop)
             s_result = self._getitem_format(s_result)
 
             return s_result
@@ -701,10 +673,7 @@ class ShardedCorpus(IndexedCorpus):
         Returns the resulting s_result.
         """
         if (result_stop - result_start) != (stop - start):
-            raise ValueError('Result start/stop range different than stop/start'
-                             'range (%d - %d vs. %d - %d)'.format(result_start,
-                                                                  result_stop,
-                                                                  start, stop))
+            raise ValueError('Result start/stop range different than stop/start range (%d - %d vs. %d - %d)'.format(result_start, result_stop, start, stop))
 
         # Dense data: just copy using numpy's slice notation
         if not self.sparse_serialization:
@@ -716,10 +685,7 @@ class ShardedCorpus(IndexedCorpus):
         # result.
         else:
             if s_result.shape != (result_start, self.dim):
-                raise ValueError('Assuption about sparse s_result shape '
-                                 'invalid: {0} expected rows, {1} real '
-                                 'rows.'.format(result_start,
-                                                s_result.shape[0]))
+                raise ValueError('Assuption about sparse s_result shape invalid: {0} expected rows, {1} real rows.'.format(result_start, s_result.shape[0]))
 
             tmp_matrix = self.current_shard[start:stop]
             s_result = sparse.vstack([s_result, tmp_matrix])
@@ -746,7 +712,7 @@ class ShardedCorpus(IndexedCorpus):
 
         """
         def row_sparse2gensim(row_idx, csr_matrix):
-            indices = csr_matrix.indices[csr_matrix.indptr[row_idx]:csr_matrix.indptr[row_idx+1]]
+            indices = csr_matrix.indices[csr_matrix.indptr[row_idx]:csr_matrix.indptr[row_idx + 1]]
             g_row = [(col_idx, csr_matrix[row_idx, col_idx]) for col_idx in indices]
             return g_row
 
@@ -783,19 +749,12 @@ class ShardedCorpus(IndexedCorpus):
         if len(args) == 0:
             args = tuple([self.output_prefix])
 
-        attrs_to_ignore = ['current_shard',
-                           'current_shard_n',
-                           'current_offset']
+        attrs_to_ignore = ['current_shard', 'current_shard_n', 'current_offset']
         if 'ignore' not in kwargs:
             kwargs['ignore'] = frozenset(attrs_to_ignore)
         else:
-            kwargs['ignore'] = frozenset([v for v in kwargs['ignore']]
-                                         + attrs_to_ignore)
+            kwargs['ignore'] = frozenset([v for v in kwargs['ignore']] + attrs_to_ignore)
         super(ShardedCorpus, self).save(*args, **kwargs)
-        #
-        # self.reset()
-        # with smart_open(self.output_prefix, 'wb') as pickle_handle:
-        #     cPickle.dump(self, pickle_handle)
 
     @classmethod
     def load(cls, fname, mmap=None):
@@ -805,8 +764,7 @@ class ShardedCorpus(IndexedCorpus):
         return super(ShardedCorpus, cls).load(fname, mmap)
 
     @staticmethod
-    def save_corpus(fname, corpus, id2word=None, progress_cnt=1000,
-                    metadata=False, **kwargs):
+    def save_corpus(fname, corpus, id2word=None, progress_cnt=1000, metadata=False, **kwargs):
         """
         Implement a serialization interface. Do not call directly;
         use the `serialize` method instead.
@@ -828,9 +786,7 @@ class ShardedCorpus(IndexedCorpus):
         ShardedCorpus(fname, corpus, **kwargs)
 
     @classmethod
-    def serialize(serializer, fname, corpus, id2word=None,
-                  index_fname=None, progress_cnt=None, labels=None,
-                  metadata=False, **kwargs):
+    def serialize(serializer, fname, corpus, id2word=None, index_fname=None, progress_cnt=None, labels=None, metadata=False, **kwargs):
         """
         Iterate through the document stream `corpus`, saving the documents
         as a ShardedCorpus to `fname`.
@@ -843,6 +799,4 @@ class ShardedCorpus(IndexedCorpus):
         Ignore the parameters id2word, index_fname, progress_cnt, labels
         and metadata. They currently do nothing and are here only to
         provide a compatible method signature with superclass."""
-        serializer.save_corpus(fname, corpus, id2word=id2word,
-                               progress_cnt=progress_cnt, metadata=metadata,
-                               **kwargs)
+        serializer.save_corpus(fname, corpus, id2word=id2word, progress_cnt=progress_cnt, metadata=metadata, **kwargs)
