@@ -21,7 +21,6 @@ from gensim.corpora import Dictionary
 from gensim.corpora import IndexedCorpus
 from gensim.matutils import MmReader
 from gensim.matutils import MmWriter
-from six import iteritems, string_types
 from six.moves import xrange
 
 
@@ -37,7 +36,7 @@ class UciReader(MmReader):
         which is expected to be in the UCI Bag-of-Words format.
         """
 
-        logger.info('Initializing corpus reader from %s' % input)
+        logger.info('Initializing corpus reader from %s', input)
 
         self.input = input
 
@@ -50,16 +49,16 @@ class UciReader(MmReader):
             except StopIteration:
                 pass
 
-        logger.info('accepted corpus with %i documents, %i features, %i non-zero entries' %
-            (self.num_docs, self.num_terms, self.num_nnz))
+        logger.info(
+            "accepted corpus with %i documents, %i features, %i non-zero entries",
+            self.num_docs, self.num_terms, self.num_nnz
+        )
 
     def skip_headers(self, input_file):
         for lineno, _ in enumerate(input_file):
             if lineno == 2:
                 break
 
-# endclass UciReader
-
 
 class UciWriter(MmWriter):
     """
@@ -110,7 +109,7 @@ class UciWriter(MmWriter):
         offsets = []
         for docno, bow in enumerate(corpus):
             if docno % progress_cnt == 0:
-                logger.info("PROGRESS: saving document #%i" % docno)
+                logger.info("PROGRESS: saving document #%i", docno)
             if index:
                 posnow = writer.fout.tell()
                 if posnow == poslast:
@@ -118,18 +117,18 @@ class UciWriter(MmWriter):
                 offsets.append(posnow)
                 poslast = posnow
 
-            vector = [(x, int(y)) for (x, y) in bow if int(y) != 0] # integer count, not floating weights
+            vector = [(x, int(y)) for (x, y) in bow if int(y) != 0]  # integer count, not floating weights
             max_id, veclen = writer.write_vector(docno, vector)
             num_terms = max(num_terms, 1 + max_id)
             num_nnz += veclen
         num_docs = docno + 1
 
         if num_docs * num_terms != 0:
-            logger.info("saved %ix%i matrix, density=%.3f%% (%i/%i)" %
-                         (num_docs, num_terms,
-                          100.0 * num_nnz / (num_docs * num_terms),
-                          num_nnz,
-                          num_docs * num_terms))
+            logger.info(
+                "saved %ix%i matrix, density=%.3f%% (%i/%i)",
+                num_docs, num_terms, 100.0 * num_nnz / (num_docs * num_terms),
+                num_nnz, num_docs * num_terms
+            )
 
         # now write proper headers, by seeking and overwriting the spaces written earlier
         writer.update_headers(num_docs, num_terms, num_nnz)
@@ -138,8 +137,6 @@ class UciWriter(MmWriter):
         if index:
             return offsets
 
-# endclass UciWriter
-
 
 class UciCorpus(UciReader, IndexedCorpus):
     """
@@ -165,7 +162,7 @@ class UciCorpus(UciReader, IndexedCorpus):
         (yielding one document at a time).
         """
         for docId, doc in super(UciCorpus, self).__iter__():
-            yield doc # get rid of docId, return the sparse vector only
+            yield doc  # get rid of docId, return the sparse vector only
 
     def create_dictionary(self):
         """
@@ -179,14 +176,14 @@ class UciCorpus(UciReader, IndexedCorpus):
         dictionary.dfs = defaultdict(int)
 
         dictionary.id2token = self.id2word
-        dictionary.token2id = dict((v, k) for k, v in iteritems(self.id2word))
+        dictionary.token2id = utils.revdict(self.id2word)
 
         dictionary.num_docs = self.num_docs
         dictionary.num_nnz = self.num_nnz
 
         for docno, doc in enumerate(self):
             if docno % 10000 == 0:
-                logger.info('PROGRESS: processing document %i of %i' % (docno, self.num_docs))
+                logger.info('PROGRESS: processing document %i of %i', docno, self.num_docs)
 
             for word, count in doc:
                 dictionary.dfs[word] += 1
@@ -214,13 +211,11 @@ class UciCorpus(UciReader, IndexedCorpus):
 
         # write out vocabulary
         fname_vocab = utils.smart_extension(fname, '.vocab')
-        logger.info("saving vocabulary of %i words to %s" % (num_terms, fname_vocab))
+        logger.info("saving vocabulary of %i words to %s", num_terms, fname_vocab)
         with utils.smart_open(fname_vocab, 'wb') as fout:
             for featureid in xrange(num_terms):
                 fout.write(utils.to_utf8("%s\n" % id2word.get(featureid, '---')))
 
-        logger.info("storing corpus in UCI Bag-of-Words format: %s" % fname)
+        logger.info("storing corpus in UCI Bag-of-Words format: %s", fname)
 
         return UciWriter.write_corpus(fname, corpus, index=True, progress_cnt=progress_cnt)
-
-# endclass UciCorpus
