@@ -15,14 +15,21 @@ Example: python -m gensim.models.lda_dispatcher
 
 
 from __future__ import with_statement
-import os, sys, logging, threading, time
+import argparse
+import os
+import sys
+import logging
+import threading
+import time
 from six import iteritems, itervalues
+
 try:
     from Queue import Queue
 except ImportError:
     from queue import Queue
 import Pyro4
 from gensim import utils
+from gensim.models.lda_worker import LDA_WORKER_PREFIX
 
 
 logger = logging.getLogger("gensim.models.lda_dispatcher")
@@ -37,8 +44,9 @@ MAX_JOBS_QUEUE = 10
 # timeout for the Queue object put/get blocking methods.
 # it should theoretically be infinity, but then keyboard interrupts don't work.
 # so this is really just a hack, see http://bugs.python.org/issue1360
-HUGE_TIMEOUT = 365 * 24 * 60 * 60 # one year
+HUGE_TIMEOUT = 365 * 24 * 60 * 60  # one year
 
+LDA_DISPATCHER_PREFIX = 'gensim.lda_dispatcher'
 
 
 class Dispatcher(object):
@@ -48,15 +56,16 @@ class Dispatcher(object):
     There should never be more than one dispatcher running at any one time.
     """
 
-    def __init__(self, maxsize=MAX_JOBS_QUEUE):
+    def __init__(self, maxsize=MAX_JOBS_QUEUE, ns_conf=None):
         """
         Note that the constructor does not fully initialize the dispatcher;
         use the `initialize()` function to populate it with workers etc.
         """
         self.maxsize = maxsize
-        self.callback = None # a pyro proxy to this object (unknown at init time, but will be set later)
-
+        self.callback = None  # a pyro proxy to this object (unknown at init time, but will be set later)
+        self.ns_conf = ns_conf if ns_conf is not None else {}
 
+    @Pyro4.expose
     def initialize(self, **model_params):
         """
         `model_params` are parameters used to initialize individual workers (gets
@@ -69,55 +78,54 @@ class Dispatcher(object):
 
         # locate all available workers and store their proxies, for subsequent RMI calls
         self.workers = {}
-        import Pyro4
-        with utils.getNS() as ns:
-            self.callback = Pyro4.Proxy('PYRONAME:gensim.lda_dispatcher') # = self
-            for name, uri in iteritems(ns.list(prefix='gensim.lda_worker')):
+        with utils.getNS(**self.ns_conf) as ns:
+            self.callback = Pyro4.Proxy(ns.list(prefix=LDA_DISPATCHER_PREFIX)[LDA_DISPATCHER_PREFIX])
+            for name, uri in iteritems(ns.list(prefix=LDA_WORKER_PREFIX)):
                 try:
                     worker = Pyro4.Proxy(uri)
                     workerid = len(self.workers)
                     # make time consuming methods work asynchronously
-                    logger.info("registering worker #%i at %s" % (workerid, uri))
+                    logger.info("registering worker #%i at %s", workerid, uri)
                     worker.initialize(workerid, dispatcher=self.callback, **model_params)
                     self.workers[workerid] = worker
                 except Pyro4.errors.PyroError:
-                    logger.warning("unresponsive worker at %s, deleting it from the name server" % uri)
+                    logger.warning("unresponsive worker at %s, deleting it from the name server", uri)
                     ns.remove(name)
 
         if not self.workers:
             raise RuntimeError('no workers found; run some lda_worker scripts on your machines first!')
 
-
+    @Pyro4.expose
     def getworkers(self):
         """
         Return pyro URIs of all registered workers.
         """
         return [worker._pyroUri for worker in itervalues(self.workers)]
 
-
+    @Pyro4.expose
     def getjob(self, worker_id):
-        logger.info("worker #%i requesting a new job" % worker_id)
+        logger.info("worker #%i requesting a new job", worker_id)
         job = self.jobs.get(block=True, timeout=1)
-        logger.info("worker #%i got a new job (%i left)" % (worker_id, self.jobs.qsize()))
+        logger.info("worker #%i got a new job (%i left)", worker_id, self.jobs.qsize())
         return job
 
-
+    @Pyro4.expose
     def putjob(self, job):
         self._jobsreceived += 1
         self.jobs.put(job, block=True, timeout=HUGE_TIMEOUT)
-        logger.info("added a new job (len(queue)=%i items)" % self.jobs.qsize())
-
+        logger.info("added a new job (len(queue)=%i items)", self.jobs.qsize())
 
+    @Pyro4.expose
     def getstate(self):
         """
         Merge states from across all workers and return the result.
         """
         logger.info("end of input, assigning all remaining jobs")
-        logger.debug("jobs done: %s, jobs received: %s" % (self._jobsdone, self._jobsreceived))
+        logger.debug("jobs done: %s, jobs received: %s", self._jobsdone, self._jobsreceived)
         while self._jobsdone < self._jobsreceived:
-            time.sleep(0.5) # check every half a second
+            time.sleep(0.5)  # check every half a second
 
-        logger.info("merging states from %i workers" % len(self.workers))
+        logger.info("merging states from %i workers", len(self.workers))
         workers = list(self.workers.values())
         result = workers[0].getstate()
         for worker in workers[1:]:
@@ -126,19 +134,19 @@ class Dispatcher(object):
         logger.info("sending out merged state")
         return result
 
-
+    @Pyro4.expose
     def reset(self, state):
         """
         Initialize all workers for a new EM iterations.
         """
         for workerid, worker in iteritems(self.workers):
-            logger.info("resetting worker %s" % workerid)
+            logger.info("resetting worker %s", workerid)
             worker.reset(state)
             worker.requestjob()
         self._jobsdone = 0
         self._jobsreceived = 0
 
-
+    @Pyro4.expose
     @Pyro4.oneway
     @utils.synchronous('lock_update')
     def jobdone(self, workerid):
@@ -150,47 +158,48 @@ class Dispatcher(object):
         and `worker.requestjob()`.
         """
         self._jobsdone += 1
-        logger.info("worker #%s finished job #%i" % (workerid, self._jobsdone))
-        self.workers[workerid].requestjob() # tell the worker to ask for another job, asynchronously (one-way)
-
+        logger.info("worker #%s finished job #%i", workerid, self._jobsdone)
+        self.workers[workerid].requestjob()  # tell the worker to ask for another job, asynchronously (one-way)
 
     def jobsdone(self):
         """Wrap self._jobsdone, needed for remote access through Pyro proxies"""
         return self._jobsdone
 
-
     @Pyro4.oneway
     def exit(self):
         """
         Terminate all registered workers and then the dispatcher.
         """
         for workerid, worker in iteritems(self.workers):
-            logger.info("terminating worker %s" % workerid)
+            logger.info("terminating worker %s", workerid)
             worker.exit()
         logger.info("terminating dispatcher")
-        os._exit(0) # exit the whole process (not just this thread ala sys.exit())
-#endclass Dispatcher
-
+        os._exit(0)  # exit the whole process (not just this thread ala sys.exit())
+# endclass Dispatcher
 
 
 def main():
-    logging.basicConfig(format = '%(asctime)s : %(levelname)s : %(message)s', level=logging.INFO)
-    logger.info("running %s" % " ".join(sys.argv))
+    parser = argparse.ArgumentParser(description=__doc__)
+    parser.add_argument("--maxsize", help="How many jobs (=chunks of N documents) to keep 'pre-fetched' in a queue (default: %(default)s)", type=int, default=MAX_JOBS_QUEUE)
+    parser.add_argument("--host", help="Nameserver hostname (default: %(default)s)", default=None)
+    parser.add_argument("--port", help="Nameserver port (default: %(default)s)", default=None, type=int)
+    parser.add_argument("--no-broadcast", help="Disable broadcast (default: %(default)s)",
+                        action='store_const', default=True, const=False)
+    parser.add_argument("--hmac", help="Nameserver hmac key (default: %(default)s)", default=None)
+    parser.add_argument('-v', '--verbose', help='Verbose flag', action='store_const', dest="loglevel", const=logging.INFO, default=logging.WARNING)
+    args = parser.parse_args()
 
-    program = os.path.basename(sys.argv[0])
-    # make sure we have enough cmd line parameters
-    if len(sys.argv) < 1:
-        print(globals()["__doc__"] % locals())
-        sys.exit(1)
-
-    if len(sys.argv) < 2:
-        maxsize = MAX_JOBS_QUEUE
-    else:
-        maxsize = int(sys.argv[1])
-    utils.pyro_daemon('gensim.lda_dispatcher', Dispatcher(maxsize=maxsize))
-
-    logger.info("finished running %s" % program)
+    logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=args.loglevel)
+    logger.info("running %s", " ".join(sys.argv))
 
+    ns_conf = {
+        "broadcast": args.no_broadcast,
+        "host": args.host,
+        "port": args.port,
+        "hmac_key": args.hmac
+    }
+    utils.pyro_daemon(LDA_DISPATCHER_PREFIX, Dispatcher(maxsize=args.maxsize, ns_conf=ns_conf), ns_conf=ns_conf)
+    logger.info("finished running %s", " ".join(sys.argv))
 
 
 if __name__ == '__main__':
