@@ -17,7 +17,9 @@ Example: python -m gensim.models.lsi_worker
 
 
 from __future__ import with_statement
-import os, sys, logging
+import os
+import sys
+import logging
 import threading
 import tempfile
 try:
@@ -31,25 +33,24 @@ from gensim import utils
 logger = logging.getLogger('gensim.models.lsi_worker')
 
 
-SAVE_DEBUG = 0 # save intermediate models after every SAVE_DEBUG updates (0 for never)
-
+SAVE_DEBUG = 0  # save intermediate models after every SAVE_DEBUG updates (0 for never)
 
 
 class Worker(object):
     def __init__(self):
         self.model = None
 
-
+    @Pyro4.expose
     def initialize(self, myid, dispatcher, **model_params):
         self.lock_update = threading.Lock()
-        self.jobsdone = 0 # how many jobs has this worker completed?
-        self.myid = myid # id of this worker in the dispatcher; just a convenience var for easy access/logging TODO remove?
+        self.jobsdone = 0  # how many jobs has this worker completed?
+        self.myid = myid  # id of this worker in the dispatcher; just a convenience var for easy access/logging TODO remove?
         self.dispatcher = dispatcher
         self.finished = False
-        logger.info("initializing worker #%s" % myid)
+        logger.info("initializing worker #%s", myid)
         self.model = lsimodel.LsiModel(**model_params)
 
-
+    @Pyro4.expose
     @Pyro4.oneway
     def requestjob(self):
         """
@@ -66,12 +67,11 @@ class Worker(object):
                 # no new job: try again, unless we're finished with all work
                 continue
         if job is not None:
-            logger.info("worker #%s received job #%i" % (self.myid, self.jobsdone))
+            logger.info("worker #%s received job #%i", self.myid, self.jobsdone)
             self.processjob(job)
             self.dispatcher.jobdone(self.myid)
         else:
-            logger.info("worker #%i stopping asking for jobs" % self.myid)
-
+            logger.info("worker #%i stopping asking for jobs", self.myid)
 
     @utils.synchronous('lock_update')
     def processjob(self, job):
@@ -81,34 +81,31 @@ class Worker(object):
             fname = os.path.join(tempfile.gettempdir(), 'lsi_worker.pkl')
             self.model.save(fname)
 
-
+    @Pyro4.expose
     @utils.synchronous('lock_update')
     def getstate(self):
-        logger.info("worker #%i returning its state after %s jobs" %
-                    (self.myid, self.jobsdone))
+        logger.info("worker #%i returning its state after %s jobs", self.myid, self.jobsdone)
         assert isinstance(self.model.projection, lsimodel.Projection)
         self.finished = True
         return self.model.projection
 
-
+    @Pyro4.expose
     @utils.synchronous('lock_update')
     def reset(self):
-        logger.info("resetting worker #%i" % self.myid)
+        logger.info("resetting worker #%i", self.myid)
         self.model.projection = self.model.projection.empty_like()
         self.finished = False
 
-
     @Pyro4.oneway
     def exit(self):
-        logger.info("terminating worker #%i" % self.myid)
+        logger.info("terminating worker #%i", self.myid)
         os._exit(0)
-#endclass Worker
-
+# endclass Worker
 
 
 def main():
-    logging.basicConfig(format = '%(asctime)s : %(levelname)s : %(message)s', level=logging.INFO)
-    logger.info("running %s" % " ".join(sys.argv))
+    logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.INFO)
+    logger.info("running %s", " ".join(sys.argv))
 
     program = os.path.basename(sys.argv[0])
     # make sure we have enough cmd line parameters
@@ -118,8 +115,7 @@ def main():
 
     utils.pyro_daemon('gensim.lsi_worker', Worker(), random_suffix=True)
 
-    logger.info("finished running %s" % program)
-
+    logger.info("finished running %s", program)
 
 
 if __name__ == '__main__':
