@@ -103,7 +103,7 @@ class PorterStemmer(object):
 
     def _doublec(self, j):
         """True <=> j,(j-1) contain a double consonant."""
-        return j > 0 and self.b[j] == self.b[j-1] and self._cons(j)
+        return j > 0 and self.b[j] == self.b[j - 1] and self._cons(j)
 
     def _cvc(self, i):
         """True <=> i-2,i-1,i has the form consonant - vowel - consonant
@@ -113,25 +113,25 @@ class PorterStemmer(object):
            cav(e), lov(e), hop(e), crim(e), but
            snow, box, tray.
         """
-        if i < 2 or not self._cons(i) or self._cons(i-1) or not self._cons(i-2):
+        if i < 2 or not self._cons(i) or self._cons(i - 1) or not self._cons(i - 2):
             return False
         return self.b[i] not in "wxy"
 
     def _ends(self, s):
         """True <=> 0,...k ends with the string s."""
-        if s[-1] != self.b[self.k]: # tiny speed-up
+        if s[-1] != self.b[self.k]:  # tiny speed-up
             return 0
         length = len(s)
         if length > (self.k + 1):
             return 0
-        if self.b[self.k-length+1:self.k+1] != s:
+        if self.b[self.k - length + 1:self.k + 1] != s:
             return 0
         self.j = self.k - length
         return 1
 
     def _setto(self, s):
         """Set (j+1),...k to the characters in the string s, adjusting k."""
-        self.b = self.b[:self.j+1] + s
+        self.b = self.b[:self.j + 1] + s
         self.k = len(self.b) - 1
 
     def _r(self, s):
@@ -171,9 +171,12 @@ class PorterStemmer(object):
                 self.k -= 1
         elif (self._ends("ed") or self._ends("ing")) and self._vowelinstem():
             self.k = self.j
-            if self._ends("at"):   self._setto("ate")
-            elif self._ends("bl"): self._setto("ble")
-            elif self._ends("iz"): self._setto("ize")
+            if self._ends("at"):
+                self._setto("ate")
+            elif self._ends("bl"):
+                self._setto("ble")
+            elif self._ends("iz"):
+                self._setto("ize")
             elif self._doublec(self.k):
                 if self.b[self.k - 1] not in "lsz":
                     self.k -= 1
@@ -193,87 +196,133 @@ class PorterStemmer(object):
         """
         ch = self.b[self.k - 1]
         if ch == 'a':
-            if self._ends("ational"):   self._r("ate")
-            elif self._ends("tional"):  self._r("tion")
+            if self._ends("ational"):
+                self._r("ate")
+            elif self._ends("tional"):
+                self._r("tion")
         elif ch == 'c':
-            if self._ends("enci"):      self._r("ence")
-            elif self._ends("anci"):    self._r("ance")
+            if self._ends("enci"):
+                self._r("ence")
+            elif self._ends("anci"):
+                self._r("ance")
         elif ch == 'e':
-            if self._ends("izer"):      self._r("ize")
+            if self._ends("izer"):
+                self._r("ize")
         elif ch == 'l':
-            if self._ends("bli"):       self._r("ble") # --DEPARTURE--
+            if self._ends("bli"):
+                self._r("ble")  # --DEPARTURE--
             # To match the published algorithm, replace this phrase with
             #   if self._ends("abli"):      self._r("able")
-            elif self._ends("alli"):    self._r("al")
-            elif self._ends("entli"):   self._r("ent")
-            elif self._ends("eli"):     self._r("e")
-            elif self._ends("ousli"):   self._r("ous")
+            elif self._ends("alli"):
+                self._r("al")
+            elif self._ends("entli"):
+                self._r("ent")
+            elif self._ends("eli"):
+                self._r("e")
+            elif self._ends("ousli"):
+                self._r("ous")
         elif ch == 'o':
-            if self._ends("ization"):   self._r("ize")
-            elif self._ends("ation"):   self._r("ate")
-            elif self._ends("ator"):    self._r("ate")
+            if self._ends("ization"):
+                self._r("ize")
+            elif self._ends("ation"):
+                self._r("ate")
+            elif self._ends("ator"):
+                self._r("ate")
         elif ch == 's':
-            if self._ends("alism"):     self._r("al")
-            elif self._ends("iveness"): self._r("ive")
-            elif self._ends("fulness"): self._r("ful")
-            elif self._ends("ousness"): self._r("ous")
+            if self._ends("alism"):
+                self._r("al")
+            elif self._ends("iveness"):
+                self._r("ive")
+            elif self._ends("fulness"):
+                self._r("ful")
+            elif self._ends("ousness"):
+                self._r("ous")
         elif ch == 't':
-            if self._ends("aliti"):     self._r("al")
-            elif self._ends("iviti"):   self._r("ive")
-            elif self._ends("biliti"):  self._r("ble")
-        elif ch == 'g': # --DEPARTURE--
-            if self._ends("logi"):      self._r("log")
+            if self._ends("aliti"):
+                self._r("al")
+            elif self._ends("iviti"):
+                self._r("ive")
+            elif self._ends("biliti"):
+                self._r("ble")
+        elif ch == 'g':  # --DEPARTURE--
+            if self._ends("logi"):
+                self._r("log")
         # To match the published algorithm, delete this phrase
 
     def _step3(self):
         """Deal with -ic-, -full, -ness etc. Similar strategy to _step2."""
         ch = self.b[self.k]
         if ch == 'e':
-            if self._ends("icate"):     self._r("ic")
-            elif self._ends("ative"):   self._r("")
-            elif self._ends("alize"):   self._r("al")
+            if self._ends("icate"):
+                self._r("ic")
+            elif self._ends("ative"):
+                self._r("")
+            elif self._ends("alize"):
+                self._r("al")
         elif ch == 'i':
-            if self._ends("iciti"):     self._r("ic")
+            if self._ends("iciti"):
+                self._r("ic")
         elif ch == 'l':
-            if self._ends("ical"):      self._r("ic")
-            elif self._ends("ful"):     self._r("")
+            if self._ends("ical"):
+                self._r("ic")
+            elif self._ends("ful"):
+                self._r("")
         elif ch == 's':
-            if self._ends("ness"):      self._r("")
+            if self._ends("ness"):
+                self._r("")
 
     def _step4(self):
         """_step4() takes off -ant, -ence etc., in context <c>vcvc<v>."""
         ch = self.b[self.k - 1]
         if ch == 'a':
-            if not self._ends("al"): return
+            if not self._ends("al"):
+                return
         elif ch == 'c':
-            if not self._ends("ance") and not self._ends("ence"): return
+            if not self._ends("ance") and not self._ends("ence"):
+                return
         elif ch == 'e':
-            if not self._ends("er"): return
+            if not self._ends("er"):
+                return
         elif ch == 'i':
-            if not self._ends("ic"): return
+            if not self._ends("ic"):
+                return
         elif ch == 'l':
-            if not self._ends("able") and not self._ends("ible"): return
+            if not self._ends("able") and not self._ends("ible"):
+                return
         elif ch == 'n':
-            if self._ends("ant"): pass
-            elif self._ends("ement"): pass
-            elif self._ends("ment"): pass
-            elif self._ends("ent"): pass
-            else: return
+            if self._ends("ant"):
+                pass
+            elif self._ends("ement"):
+                pass
+            elif self._ends("ment"):
+                pass
+            elif self._ends("ent"):
+                pass
+            else:
+                return
         elif ch == 'o':
-            if self._ends("ion") and self.b[self.j] in "st": pass
-            elif self._ends("ou"): pass
+            if self._ends("ion") and self.b[self.j] in "st":
+                pass
+            elif self._ends("ou"):
+                pass
             # takes care of -ous
-            else: return
+            else:
+                return
         elif ch == 's':
-            if not self._ends("ism"): return
+            if not self._ends("ism"):
+                return
         elif ch == 't':
-            if not self._ends("ate") and not self._ends("iti"): return
+            if not self._ends("ate") and not self._ends("iti"):
+                return
         elif ch == 'u':
-            if not self._ends("ous"): return
+            if not self._ends("ous"):
+                return
         elif ch == 'v':
-            if not self._ends("ive"): return
+            if not self._ends("ive"):
+                return
         elif ch == 'z':
-            if not self._ends("ize"): return
+            if not self._ends("ize"):
+                return
         else:
             return
         if self._m() > 1:
@@ -295,7 +344,7 @@ class PorterStemmer(object):
         w = w.lower()
         k = len(w) - 1
         if k <= 1:
-            return w # --DEPARTURE--
+            return w  # --DEPARTURE--
 
         # With this line, strings of length 1 or 2 don't go through the
         # stemming process, although no mention is made of this in the
@@ -311,13 +360,13 @@ class PorterStemmer(object):
         self._step3()
         self._step4()
         self._step5()
-        return self.b[:self.k+1]
+        return self.b[:self.k + 1]
 
     def stem_sentence(self, txt):
-        return " ".join(map(self.stem, txt.split()))
+        return " ".join(self.stem(x) for x in txt.split())
 
     def stem_documents(self, docs):
-        return map(self.stem_sentence, docs)
+        return [self.stem_sentence(x) for x in docs]
 
 
 if __name__ == '__main__':
