@@ -23,8 +23,8 @@ How It Works
 The `Similarity` class splits the index into several smaller sub-indexes ("shards"),
 which are disk-based. If your entire index fits in memory (~hundreds of thousands
 documents for 1GB of RAM), you can also use the `MatrixSimilarity` or `SparseMatrixSimilarity`
-classes directly. These are more simple but do not scale well (the entire index is
-kept in RAM).
+classes directly. These are more simple but do not scale as well (they keep the
+entire index in RAM, no sharding).
 
 Once the index has been initialized, you can query for document similarity simply by:
 
@@ -42,12 +42,11 @@ To see the speed-up on your machine, run ``python -m gensim.test.simspeed``
 
 There is also a special syntax for when you need similarity of documents in the index
 to the index itself (i.e. queries=indexed documents themselves). This special syntax
-already uses the faster, batch queries internally:
+uses the faster, batch queries internally and **is ideal for all-vs-all pairwise similarities**:
 
->>> for similarities in index: # return all similarities of the 1st index document, then 2nd...
+>>> for similarities in index: # yield similarities of the 1st indexed document, then 2nd...
 >>>     ...
 
-
 """
 
 
@@ -63,7 +62,7 @@ from gensim import interfaces, utils, matutils
 from six.moves import map as imap, xrange, zip as izip
 
 
-logger = logging.getLogger('gensim.similarities.docsim')
+logger = logging.getLogger(__name__)
 
 PARALLEL_SHARDS = False
 try:
@@ -83,11 +82,12 @@ class Shard(utils.SaveLoad):
     request (query).
 
     """
+
     def __init__(self, fname, index):
         self.dirname, self.fname = os.path.split(fname)
         self.length = len(index)
         self.cls = index.__class__
-        logger.info("saving index shard to %s" % self.fullname())
+        logger.info("saving index shard to %s", self.fullname())
         index.save(self.fullname())
         self.index = self.get_index()
 
@@ -105,16 +105,14 @@ class Shard(utils.SaveLoad):
         return result
 
     def __str__(self):
-        return ("%s Shard(%i documents in %s)" % (self.cls.__name__, len(self), self.fullname()))
-
+        return "%s Shard(%i documents in %s)" % (self.cls.__name__, len(self), self.fullname())
 
     def get_index(self):
         if not hasattr(self, 'index'):
-            logger.debug("mmaping index from %s" % self.fullname())
+            logger.debug("mmaping index from %s", self.fullname())
             self.index = self.cls.load(self.fullname(), mmap='r')
         return self.index
 
-
     def get_document_id(self, pos):
         """Return index vector at position `pos`.
 
@@ -124,26 +122,24 @@ class Shard(utils.SaveLoad):
         assert 0 <= pos < len(self), "requested position out of range"
         return self.get_index().index[pos]
 
-
     def __getitem__(self, query):
         index = self.get_index()
         try:
             index.num_best = self.num_best
             index.normalize = self.normalize
-        except:
+        except Exception:
             raise ValueError("num_best and normalize have to be set before querying a proxy Shard object")
         return index[query]
 
 
 def query_shard(args):
-    query, shard = args # simulate starmap (not part of multiprocessing in older Pythons)
-    logger.debug("querying shard %s num_best=%s in process %s" % (shard, shard.num_best, os.getpid()))
+    query, shard = args  # simulate starmap (not part of multiprocessing in older Pythons)
+    logger.debug("querying shard %s num_best=%s in process %s", shard, shard.num_best, os.getpid())
     result = shard[query]
-    logger.debug("finished querying shard %s in process %s" % (shard, os.getpid()))
+    logger.debug("finished querying shard %s in process %s", shard, os.getpid())
     return result
 
 
-
 class Similarity(interfaces.SimilarityABC):
     """
     Compute cosine similarity of a dynamic query against a static corpus of documents
@@ -154,7 +150,8 @@ class Similarity(interfaces.SimilarityABC):
     The shards themselves are simply stored as files to disk and mmap'ed back as needed.
 
     """
-    def __init__(self, output_prefix, corpus, num_features, num_best=None, chunksize=256, shardsize=32768):
+
+    def __init__(self, output_prefix, corpus, num_features, num_best=None, chunksize=256, shardsize=32768, norm='l2'):
         """
         Construct the index from `corpus`. The index can be later extended by calling
         the `add_documents` method. **Note**: documents are split (internally, transparently)
@@ -169,6 +166,8 @@ class Similarity(interfaces.SimilarityABC):
         `num_features` is the number of features in the `corpus` (e.g. size of the
         dictionary, or the number of latent topics for latent semantic models).
 
+        `norm` is the user-chosen normalization to use. Accepted values are: 'l1' and 'l2'.
+
         If `num_best` is left unspecified, similarity queries will return a full
         vector with one float for every document in the index:
 
@@ -194,10 +193,10 @@ class Similarity(interfaces.SimilarityABC):
             self.output_prefix = utils.randfname(prefix='simserver')
         else:
             self.output_prefix = output_prefix
-        logger.info("starting similarity index under %s" % self.output_prefix)
+        logger.info("starting similarity index under %s", self.output_prefix)
         self.num_features = num_features
         self.num_best = num_best
-        self.normalize = True
+        self.norm = norm
         self.chunksize = int(chunksize)
         self.shardsize = shardsize
         self.shards = []
@@ -206,15 +205,13 @@ class Similarity(interfaces.SimilarityABC):
         if corpus is not None:
             self.add_documents(corpus)
 
-
     def __len__(self):
         return len(self.fresh_docs) + sum([len(shard) for shard in self.shards])
 
-
     def __str__(self):
-        return ("Similarity index with %i documents in %i shards (stored under %s)" %
-                (len(self), len(self.shards), self.output_prefix))
-
+        return "Similarity index with %i documents in %i shards (stored under %s)" % (
+            len(self), len(self.shards), self.output_prefix
+        )
 
     def add_documents(self, corpus):
         """
@@ -223,7 +220,7 @@ class Similarity(interfaces.SimilarityABC):
         Internally, documents are buffered and then spilled to disk when there's
         `self.shardsize` of them (or when a query is issued).
         """
-        min_ratio = 1.0 # 0.5 to only reopen shards that are <50% complete
+        min_ratio = 1.0  # 0.5 to only reopen shards that are <50% complete
         if self.shards and len(self.shards[-1]) < min_ratio * self.shardsize:
             # The last shard was incomplete (<; load it back and add the documents there, don't start a new shard
             self.reopen_shard()
@@ -235,16 +232,15 @@ class Similarity(interfaces.SimilarityABC):
             else:
                 doclen = len(doc)
                 if doclen < 0.3 * self.num_features:
-                    doc = matutils.unitvec(matutils.corpus2csc([doc], self.num_features).T)
+                    doc = matutils.unitvec(matutils.corpus2csc([doc], self.num_features).T, self.norm)
                 else:
-                    doc = matutils.unitvec(matutils.sparse2full(doc, self.num_features))
+                    doc = matutils.unitvec(matutils.sparse2full(doc, self.num_features), self.norm)
             self.fresh_docs.append(doc)
             self.fresh_nnz += doclen
             if len(self.fresh_docs) >= self.shardsize:
                 self.close_shard()
             if len(self.fresh_docs) % 10000 == 0:
-                logger.info("PROGRESS: fresh_shard size=%i" % len(self.fresh_docs))
-
+                logger.info("PROGRESS: fresh_shard size=%i", len(self.fresh_docs))
 
     def shardid2filename(self, shardid):
         if self.output_prefix.endswith('.'):
@@ -252,7 +248,6 @@ class Similarity(interfaces.SimilarityABC):
         else:
             return "%s.%s" % (self.output_prefix, shardid)
 
-
     def close_shard(self):
         """
         Force the latest shard to close (be converted to a matrix and stored
@@ -268,32 +263,31 @@ class Similarity(interfaces.SimilarityABC):
         # consider the shard sparse if its density is < 30%
         issparse = 0.3 > 1.0 * self.fresh_nnz / (len(self.fresh_docs) * self.num_features)
         if issparse:
-            index = SparseMatrixSimilarity(self.fresh_docs, num_terms=self.num_features,
-                                           num_docs=len(self.fresh_docs), num_nnz=self.fresh_nnz)
+            index = SparseMatrixSimilarity(
+                self.fresh_docs, num_terms=self.num_features, num_docs=len(self.fresh_docs), num_nnz=self.fresh_nnz
+            )
         else:
             index = MatrixSimilarity(self.fresh_docs, num_features=self.num_features)
-        logger.info("creating %s shard #%s" % ('sparse' if issparse else 'dense', shardid))
+        logger.info("creating %s shard #%s", 'sparse' if issparse else 'dense', shardid)
         shard = Shard(self.shardid2filename(shardid), index)
         shard.num_best = self.num_best
         shard.num_nnz = self.fresh_nnz
         self.shards.append(shard)
         self.fresh_docs, self.fresh_nnz = [], 0
 
-
     def reopen_shard(self):
         assert self.shards
         if self.fresh_docs:
             raise ValueError("cannot reopen a shard with fresh documents in index")
         last_shard = self.shards[-1]
         last_index = last_shard.get_index()
-        logger.info("reopening an incomplete shard of %i documents" % len(last_shard))
+        logger.info("reopening an incomplete shard of %i documents", len(last_shard))
 
         self.fresh_docs = list(last_index.index)
         self.fresh_nnz = last_shard.num_nnz
-        del self.shards[-1] # remove the shard from index, *but its file on disk is not deleted*
+        del self.shards[-1]  # remove the shard from index, *but its file on disk is not deleted*
         logger.debug("reopen complete")
 
-
     def query_shards(self, query):
         """
         Return the result of applying shard[query] for each shard in self.shards,
@@ -304,7 +298,7 @@ class Similarity(interfaces.SimilarityABC):
         """
         args = zip([query] * len(self.shards), self.shards)
         if PARALLEL_SHARDS and PARALLEL_SHARDS > 1:
-            logger.debug("spawning %i query processes" % PARALLEL_SHARDS)
+            logger.debug("spawning %i query processes", PARALLEL_SHARDS)
             pool = multiprocessing.Pool(PARALLEL_SHARDS)
             result = pool.imap(query_shard, args, chunksize=1 + len(args) / PARALLEL_SHARDS)
         else:
@@ -313,7 +307,6 @@ class Similarity(interfaces.SimilarityABC):
             result = imap(query_shard, args)
         return pool, result
 
-
     def __getitem__(self, query):
         """Get similarities of document `query` to all documents in the corpus.
 
@@ -323,12 +316,12 @@ class Similarity(interfaces.SimilarityABC):
         of all query documents vs. all corpus document. This batch query is more
         efficient than computing the similarities one document after another.
         """
-        self.close_shard() # no-op if no documents added to index since last query
+        self.close_shard()  # no-op if no documents added to index since last query
 
         # reset num_best and normalize parameters, in case they were changed dynamically
         for shard in self.shards:
             shard.num_best = self.num_best
-            shard.normalize = self.normalize
+            shard.normalize = self.norm
 
         # there are 4 distinct code paths, depending on whether input `query` is
         # a corpus (or numpy/scipy matrix) or a single document, and whether the
@@ -343,8 +336,7 @@ class Similarity(interfaces.SimilarityABC):
             # the following uses a lot of lazy evaluation and (optionally) parallel
             # processing, to improve query latency and minimize memory footprint.
             offsets = numpy.cumsum([0] + [len(shard) for shard in self.shards])
-            convert = lambda doc, shard_no: [(doc_index + offsets[shard_no], sim)
-                                             for doc_index, sim in doc]
+            convert = lambda doc, shard_no: [(doc_index + offsets[shard_no], sim) for doc_index, sim in doc]
             is_corpus, query = utils.is_corpus(query)
             is_corpus = is_corpus or hasattr(query, 'ndim') and query.ndim > 1 and query.shape[0] > 1
             if not is_corpus:
@@ -368,43 +360,39 @@ class Similarity(interfaces.SimilarityABC):
 
         return result
 
-
     def vector_by_id(self, docpos):
         """
         Return indexed vector corresponding to the document at position `docpos`.
         """
-        self.close_shard() # no-op if no documents added to index since last query
+        self.close_shard()  # no-op if no documents added to index since last query
         pos = 0
         for shard in self.shards:
             pos += len(shard)
             if docpos < pos:
                 break
         if not self.shards or docpos < 0 or docpos >= pos:
-            raise ValueError("invalid document position: %s (must be 0 <= x < %s)" %
-                             (docpos, len(self)))
+            raise ValueError("invalid document position: %s (must be 0 <= x < %s)" % (docpos, len(self)))
         result = shard.get_document_id(docpos - pos + len(shard))
         return result
 
-
     def similarity_by_id(self, docpos):
         """
         Return similarity of the given document only. `docpos` is the position
         of the query document within index.
         """
         query = self.vector_by_id(docpos)
-        norm, self.normalize = self.normalize, False
+        norm, self.norm = self.norm, False
         result = self[query]
-        self.normalize = norm
+        self.norm = norm
         return result
 
-
     def __iter__(self):
         """
         For each index document, compute cosine similarity against all other
         documents in the index and yield the result.
         """
         # turn off query normalization (vectors in the index are already normalized, save some CPU)
-        norm, self.normalize = self.normalize, False
+        norm, self.norm = self.norm, False
 
         for chunk in self.iter_chunks():
             if chunk.shape[0] > 1:
@@ -413,8 +401,7 @@ class Similarity(interfaces.SimilarityABC):
             else:
                 yield self[chunk]
 
-        self.normalize = norm # restore normalization
-
+        self.norm = norm  # restore normalization
 
     def iter_chunks(self, chunksize=None):
         """
@@ -437,10 +424,9 @@ class Similarity(interfaces.SimilarityABC):
                 # (unlike numpy). so, clip the end of the chunk explicitly to make
                 # scipy.sparse happy
                 chunk_end = min(query.shape[0], chunk_start + chunksize)
-                chunk = query[chunk_start: chunk_end] # create a view
+                chunk = query[chunk_start: chunk_end]  # create a view
                 yield chunk
 
-
     def check_moved(self):
         """
         Update shard locations, in case the server directory has moved on filesystem.
@@ -449,7 +435,6 @@ class Similarity(interfaces.SimilarityABC):
         for shard in self.shards:
             shard.dirname = dirname
 
-
     def save(self, fname=None, *args, **kwargs):
         """
         Save the object via pickling (also see load) under filename specified in
@@ -471,9 +456,8 @@ class Similarity(interfaces.SimilarityABC):
         """
         import glob
         for fname in glob.glob(self.output_prefix + '*'):
-            logger.info("deleting %s" % fname)
+            logger.info("deleting %s", fname)
             os.remove(fname)
-#endclass Similarity
 
 
 class MatrixSimilarity(interfaces.SimilarityABC):
@@ -490,6 +474,7 @@ class MatrixSimilarity(interfaces.SimilarityABC):
     See also `Similarity` and `SparseMatrixSimilarity` in this module.
 
     """
+
     def __init__(self, corpus, num_best=None, dtype=numpy.float32, num_features=None, chunksize=256, corpus_len=None):
         """
         `num_features` is the number of features in the corpus (will be determined
@@ -510,7 +495,10 @@ class MatrixSimilarity(interfaces.SimilarityABC):
 
         if corpus is not None:
             if self.num_features <= 0:
-                raise ValueError("cannot index a corpus with zero features (you must specify either `num_features` or a non-empty corpus in the constructor)")
+                raise ValueError(
+                    "cannot index a corpus with zero features (you must specify either `num_features` "
+                    "or a non-empty corpus in the constructor)"
+                )
             logger.info("creating matrix with %i documents and %i features", corpus_len, num_features)
             self.index = numpy.empty(shape=(corpus_len, num_features), dtype=dtype)
             # iterate over corpus, populating the numpy index matrix with (normalized)
@@ -549,7 +537,8 @@ class MatrixSimilarity(interfaces.SimilarityABC):
         if is_corpus:
             query = numpy.asarray(
                 [matutils.sparse2full(vec, self.num_features) for vec in query],
-                dtype=self.index.dtype)
+                dtype=self.index.dtype
+            )
         else:
             if scipy.sparse.issparse(query):
                 query = query.toarray()  # convert sparse to dense
@@ -567,7 +556,90 @@ class MatrixSimilarity(interfaces.SimilarityABC):
 
     def __str__(self):
         return "%s<%i docs, %i features>" % (self.__class__.__name__, len(self), self.index.shape[1])
-#endclass MatrixSimilarity
+
+
+class WmdSimilarity(interfaces.SimilarityABC):
+    """
+    Document similarity (like MatrixSimilarity) that uses the negative of WMD
+    as a similarity measure. See gensim.models.word2vec.wmdistance for more
+    information.
+
+    When a `num_best` value is provided, only the most similar documents are
+    retrieved.
+
+    When using this code, please consider citing the following papers:
+
+    .. Ofir Pele and Michael Werman, "A linear time histogram metric for improved SIFT matching".
+    .. Ofir Pele and Michael Werman, "Fast and robust earth mover's distances".
+    .. Matt Kusner et al. "From Word Embeddings To Document Distances".
+
+    Example:
+        # See Tutorial Notebook for more examples https://github.com/RaRe-Technologies/gensim/blob/develop/docs/notebooks/WMD_tutorial.ipynb
+        >>> # Given a document collection "corpus", train word2vec model.
+        >>> model = word2vec(corpus)
+        >>> instance = WmdSimilarity(corpus, model, num_best=10)
+        >>> # Make query.
+        >>> query = 'Very good, you should seat outdoor.'
+        >>> sims = instance[query]
+    """
+
+    def __init__(self, corpus, w2v_model, num_best=None, normalize_w2v_and_replace=True, chunksize=256):
+        """
+        corpus:                         List of lists of strings, as in gensim.models.word2vec.
+        w2v_model:                      A trained word2vec model.
+        num_best:                       Number of results to retrieve.
+        normalize_w2v_and_replace:      Whether or not to normalize the word2vec vectors to length 1.
+        """
+        self.corpus = corpus
+        self.w2v_model = w2v_model
+        self.num_best = num_best
+        self.chunksize = chunksize
+
+        # Normalization of features is not possible, as corpus is a list (of lists) of strings.
+        self.normalize = False
+
+        # index is simply an array from 0 to size of corpus.
+        self.index = numpy.array(range(len(corpus)))
+
+        if normalize_w2v_and_replace:
+            # Normalize vectors in word2vec class to length 1.
+            w2v_model.init_sims(replace=True)
+
+    def __len__(self):
+        return len(self.corpus)
+
+    def get_similarities(self, query):
+        """
+        **Do not use this function directly; use the self[query] syntax instead.**
+        """
+        if isinstance(query, numpy.ndarray):
+            # Convert document indexes to actual documents.
+            query = [self.corpus[i] for i in query]
+
+        if not isinstance(query[0], list):
+            query = [query]
+
+        n_queries = len(query)
+        result = []
+        for qidx in range(n_queries):
+            # Compute similarity for each query.
+            qresult = [self.w2v_model.wmdistance(document, query[qidx]) for document in self.corpus]
+            qresult = numpy.array(qresult)
+            qresult = 1. / (1. + qresult)  # Similarity is the negative of the distance.
+
+            # Append single query result to list of all results.
+            result.append(qresult)
+
+        if len(result) == 1:
+            # Only one query.
+            result = result[0]
+        else:
+            result = numpy.array(result)
+
+        return result
+
+    def __str__(self):
+        return "%s<%i docs, %i features>" % (self.__class__.__name__, len(self), self.w2v_model.wv.syn0.shape[1])
 
 
 class SparseMatrixSimilarity(interfaces.SimilarityABC):
@@ -581,13 +653,19 @@ class SparseMatrixSimilarity(interfaces.SimilarityABC):
     The matrix is internally stored as a `scipy.sparse.csr` matrix. Unless the entire
     matrix fits into main memory, use `Similarity` instead.
 
+    Takes an optional `maintain_sparsity` argument, setting this to True
+    causes `get_similarities` to return a sparse matrix instead of a
+    dense representation if possible.
+
     See also `Similarity` and `MatrixSimilarity` in this module.
     """
+
     def __init__(self, corpus, num_features=None, num_terms=None, num_docs=None, num_nnz=None,
-                 num_best=None, chunksize=500, dtype=numpy.float32):
+                 num_best=None, chunksize=500, dtype=numpy.float32, maintain_sparsity=False):
         self.num_best = num_best
         self.normalize = True
         self.chunksize = chunksize
+        self.maintain_sparsity = maintain_sparsity
 
         if corpus is not None:
             logger.info("creating sparse index")
@@ -610,18 +688,18 @@ class SparseMatrixSimilarity(interfaces.SimilarityABC):
             corpus = (matutils.scipy2sparse(v) if scipy.sparse.issparse(v) else
                       (matutils.full2sparse(v) if isinstance(v, numpy.ndarray) else
                        matutils.unitvec(v)) for v in corpus)
-            self.index = matutils.corpus2csc(corpus, num_terms=num_terms, num_docs=num_docs, num_nnz=num_nnz,
-                                              dtype=dtype, printprogress=10000).T
+            self.index = matutils.corpus2csc(
+                corpus, num_terms=num_terms, num_docs=num_docs, num_nnz=num_nnz,
+                dtype=dtype, printprogress=10000
+            ).T
 
             # convert to Compressed Sparse Row for efficient row slicing and multiplications
-            self.index = self.index.tocsr() # currently no-op, CSC.T is already CSR
-            logger.info("created %r" % self.index)
-
+            self.index = self.index.tocsr()  # currently no-op, CSC.T is already CSR
+            logger.info("created %r", self.index)
 
     def __len__(self):
         return self.index.shape[0]
 
-
     def get_similarities(self, query):
         """
         Return similarity of sparse vector `query` to all documents in the corpus,
@@ -639,7 +717,7 @@ class SparseMatrixSimilarity(interfaces.SimilarityABC):
             query = matutils.corpus2csc(query, self.index.shape[1], dtype=self.index.dtype)
         else:
             if scipy.sparse.issparse(query):
-                query = query.T # convert documents=rows to documents=columns
+                query = query.T  # convert documents=rows to documents=columns
             elif isinstance(query, numpy.ndarray):
                 if query.ndim == 1:
                     query.shape = (1, len(query))
@@ -649,12 +727,14 @@ class SparseMatrixSimilarity(interfaces.SimilarityABC):
                 query = matutils.corpus2csc([query], self.index.shape[1], dtype=self.index.dtype)
 
         # compute cosine similarity against every other document in the collection
-        result = self.index * query.tocsc() # N x T * T x C = N x C
+        result = self.index * query.tocsc()  # N x T * T x C = N x C
         if result.shape[1] == 1 and not is_corpus:
             # for queries of one document, return a 1d array
             result = result.toarray().flatten()
+        elif self.maintain_sparsity:
+            # avoid converting to dense array if maintaining sparsity
+            result = result.T
         else:
             # otherwise, return a 2d matrix (#queries x #index)
             result = result.toarray().T
         return result
-#endclass SparseMatrixSimilarity
