@@ -10,13 +10,14 @@ from gensim.summarization.commons import build_graph as _build_graph
 from gensim.summarization.commons import remove_unreachable_nodes as _remove_unreachable_nodes
 from gensim.summarization.bm25 import get_bm25_weights as _bm25_weights
 from gensim.corpora import Dictionary
-from scipy.sparse import csr_matrix
 from math import log10 as _log10
 from six.moves import xrange
 
 
 INPUT_MIN_LENGTH = 10
 
+WEIGHT_THRESHOLD = 1.e-3
+
 logger = logging.getLogger(__name__)
 
 
@@ -26,7 +27,7 @@ def _set_graph_edge_weights(graph):
 
     for i in xrange(len(documents)):
         for j in xrange(len(documents)):
-            if i == j:
+            if i == j or weights[i][j] < WEIGHT_THRESHOLD:
                 continue
 
             sentence_1 = documents[i]
@@ -131,32 +132,37 @@ def summarize_corpus(corpus, ratio=0.2):
     """
     Returns a list of the most important documents of a corpus using a
     variation of the TextRank algorithm.
-    The input must have at least INPUT_MIN_LENGTH documents for the
+    The input must have at least INPUT_MIN_LENGTH (%d) documents for the
     summary to make sense.
 
     The length of the output can be specified using the ratio parameter,
     which determines how many documents will be chosen for the summary
-    (defaults at 20% of the number of documents of the corpus).
+    (defaults at 20%% of the number of documents of the corpus).
 
     The most important documents are returned as a list sorted by the
     document score, highest first.
-
-    """
+    """ % INPUT_MIN_LENGTH
     hashable_corpus = _build_hasheable_corpus(corpus)
 
     # If the corpus is empty, the function ends.
     if len(corpus) == 0:
         logger.warning("Input corpus is empty.")
-        return
+        return []
 
     # Warns the user if there are too few documents.
     if len(corpus) < INPUT_MIN_LENGTH:
-        logger.warning("Input corpus is expected to have at least " + str(INPUT_MIN_LENGTH) + " documents.")
+        logger.warning("Input corpus is expected to have at least %d documents.", INPUT_MIN_LENGTH)
 
     graph = _build_graph(hashable_corpus)
     _set_graph_edge_weights(graph)
     _remove_unreachable_nodes(graph)
 
+    # Cannot calculate eigenvectors if number of unique documents in corpus < 3.
+    # Warns user to add more text. The function ends.
+    if len(graph.nodes()) < 3:
+        logger.warning("Please add more sentences to the text. The number of reachable nodes is below 3")
+        return []
+
     pagerank_scores = _pagerank(graph)
 
     hashable_corpus.sort(key=lambda doc: pagerank_scores.get(doc, 0), reverse=True)
@@ -178,11 +184,13 @@ def summarize(text, ratio=0.2, word_count=None, split=False):
 
     The length of the output can be specified using the ratio and
     word_count parameters:
+
         ratio should be a number between 0 and 1 that determines the
-    percentage of the number of sentences of the original text to be
-    chosen for the summary (defaults at 0.2).
+        percentage of the number of sentences of the original text to be
+        chosen for the summary (defaults at 0.2).
         word_count determines how many words will the output contain.
-    If both parameters are provided, the ratio will be ignored.
+        If both parameters are provided, the ratio will be ignored.
+
     """
     # Gets a list of processed sentences.
     sentences = _clean_text_by_sentences(text)
@@ -190,16 +198,25 @@ def summarize(text, ratio=0.2, word_count=None, split=False):
     # If no sentence could be identified, the function ends.
     if len(sentences) == 0:
         logger.warning("Input text is empty.")
-        return
+        return [] if split else u""
+
+    # If only one sentence is present, the function raises an error (Avoids ZeroDivisionError).
+    if len(sentences) == 1:
+        raise ValueError("input must have more than one sentence")
 
     # Warns if the text is too short.
     if len(sentences) < INPUT_MIN_LENGTH:
-        logger.warning("Input text is expected to have at least " + str(INPUT_MIN_LENGTH) + " sentences.")
+        logger.warning("Input text is expected to have at least %d sentences.", INPUT_MIN_LENGTH)
 
     corpus = _build_corpus(sentences)
 
     most_important_docs = summarize_corpus(corpus, ratio=ratio if word_count is None else 1)
 
+    # If couldn't get important docs, the algorithm ends.
+    if not most_important_docs:
+        logger.warning("Couldn't get relevant sentences.")
+        return [] if split else u""
+
     # Extracts the most important sentences with the selected criterion.
     extracted_sentences = _extract_important_sentences(sentences, corpus, most_important_docs, word_count)
 
